"""
Movie module (imdb package).

This module provides the Movie class, used to store informations on
a given movie.

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import copy
from imdb.utils import imdbURL_main, imdbURL_combdet, imdbURL_plot


class Movie:
    """A Movie.
    
    A Movie object emulates (most of) the dictionary interface.

    Every information about a movie can be accessed as:
        movieObject['information']
    to get a list of the kind of informations stored in a
    Movie object, use the keys() method; some useful aliases
    are defined (as "plot summary" for the "plot" key); see the
    keys_alias dictionary.
    """
    # Aliases for some not-so-intuitive keys.
    keys_alias = {
                'user rating':  'rating',
                'plot summary': 'plot',
                'plot summaries': 'plot',
                'directed by':  'director',
                'writting credits': 'writer',
                'produced by':  'producer',
                'original music by':    'composer',
                'original music':    'composer',
                'non-original music by':    'composer',
                'non-original music':    'composer',
                'music':    'composer',
                'cinematography by':    'cinematographer',
                'cinematography':   'cinematographer',
                'film editing by':  'editor',
                'film editing': 'editor',
                'editing':  'editor',
                'actors':   'cast',
                'casting by':   'casting director',
                'casting':  'casting director',
                'art direction by': 'art director',
                'art direction': 'art director',
                'set decoration by':    'set decorator',
                'set decoration':   'set decorator',
                'costume design by':    'costume designer',
                'costume design':    'costume designer',
                'makeup department':    'make up',
                'makeup':    'make up',
                'make-up':    'make up',
                'production management':    'production manager',
                'second unit director or assistant director':
                                                'assistant director',
                'second unit director':   'assistant director',
                'assistant director':   'assistant director',
                'sound department': 'sound crew',
                'special effects by':   'special effects',
                'visual effects by':    'visual effects',
                'stunts':   'stunt performer',
                'other crew':   'crewmembers',
                'other Companies':  'miscellaneous companies',
                'aka':  'akas',
                'also known as':    'akas',
                'country':  'countries',
                'runtime':  'runtimes',
                'language': 'languages',
                'certificate':  'certificates',
                'certifications':   'certificates',
                'certification':    'certificates'}

    def __init__(self, imdbID=None, myTitle='', myID=None, movieData={}):
        """Initialize a Movie object.
        
        *imdbID* -- the unique identifier for the movie.
        *myTitle* -- your personal title for the movie.
        *myID* -- your personal identifier for the movie.
        *movieData* -- a dictionary used to initialize the object.
        """
        self.reset()
        self.set_movie_data(movieData, override=1)
        self.set_myTitle(myTitle)
        self.set_myID(myID)
        self.set_imdbID(imdbID)

    def reset(self):
        """Reset the Movie object."""
        self.imdbID = None
        self.__movie_data = {}
        self.myTitle = ''
        self.myID = None
        self.imdbURL = ''
        self.imdbURL_combined = ''
        self.imdbURL_plotsummary = ''

    def set_myTitle(self, myTitle):
        """Set the personal title for the movie.
        This can be the title in your native language, or an alias
        you use for this movie."""
        self.myTitle = myTitle

    def get_myTitle(self):
        """Return the personal title."""
        return self.myTitle

    def set_myID(self, myID):
        """Set the personal ID for the movie.
        This can be an ID you use to identify this movie or the VHS,
        DVD, Movix file were the movie is stored."""
        self.myID = myID
    
    def get_myID(self):
        """Return the personal ID."""
        return self.myID

    def set_imdbID(self, imdbID):
        """Set the imdbID for the movie.
        It must be the 7-ciphers numbers used by the IMDb's web server."""
        if imdbID is None:
            return
        imdbID = str('%.07i' % int(imdbID))
        self.imdbID = imdbID
        self.imdbURL = imdbURL_main % self.imdbID
        self.imdbURL_combined = imdbURL_combdet % self.imdbURL
        self.imdbURL_plotsummary = imdbURL_plot % self.imdbURL
    
    def get_imdbID(self):
        """Return the imdbID."""
        return self.imdbID

    def get_imdbURL(self):
        """Return the URL for the main web page of the movie."""
        return self.imdbURL

    def get_imdbURL_combinedcredits(self):
        """Return the URL for the "combined details" web page of the movie."""
        return self.imdbURL_combined

    def get_imdbURL_plotsummary(self):
        """Return the URL for the "plot summary" web page of the movie."""
        return self.imdbURL_plotsummary

    def set_movie_data(self, md, override=0):
        """Set the movie data to the given dictionary; if 'override' is
        set, the previous data is removed, otherwise the two dictionary
        are merged."""
        # XXX: uh.  Not sure this the best place/way to do it.
        md = copy.deepcopy(md)
        if not override:
            self.__movie_data.update(md)
        else:
            self.__movie_data = md.copy()
    
    def get_movie_data(self):
        """Return the whole dictionary."""
        return self.__movie_data

    def clear(self):
        """Reset the dictionary."""
        self.__movie_data = {}

    def has_key(self, key):
        """Return true if a given section is defined."""
        return self.__movie_data.has_key(key)

    def keys(self):
        """Return a list of valid keys."""
        l = self.__movie_data.keys() + ['imdbID', 'myTitle', 'myID']
        if self.__movie_data.get('title'):
            l += ['year', 'imdbIndex', 'kind', 'long imdb title']
        return l

    def items(self):
        """Return the items in the dictionary."""
        return self.__movie_data.items()

    def values(self):
        """Return the values in the dictionary."""
        return self.__movie_data.values()

    def append_item(self, key, item):
        """The item is appended to the list identified by
        the given key."""
        # TODO: this and the two other methods below are here only
        #       for _future_ usage, when it will make sense to modify
        #       a Movie object; right now they're incomplete and should
        #       not be used.
        if not self.__movie_data.has_key(key):
            self.__movie_data[key] = []
        self.__movie_data[key].append(item)

    def set_item(self, key, item):
        """Directly store the item with the given key."""
        self.__movie_data[key] = item

    def __setitem__(self, key, item):
        """Directly store the item with the given key."""
        self.__movie_data[key] = item
    
    def get(self, key, default=None):
        """Return the given section, or None if it's not found."""
        try:
            return self.__getitem__(key)
        except KeyError:
            return default

    def __getitem__(self, key):
        """Return the value for a given key, checking key aliases;
        a KeyError exception is raised if the key is not found."""
        # XXX: internally the title, year, imdbIndex, kind and
        #      "long imdb title" are stored in the self.__movie_data
        #      dictionary within the 'title' key, using a MovieTitle object;
        #      here, movie['title'] doesn't return the MovieTitle object,
        #      but the "usual" title, as a string.
        if key == 'title':
            return self.__movie_data['title']['title']
        elif key == 'year':
            return self.__movie_data['title']['year']
        elif key == 'imdbIndex':
            return self.__movie_data['title']['imdbIndex']
        elif key == 'kind':
            return self.__movie_data['title']['kind']
        elif key == 'long imdb title':
            return self.__movie_data['title'].get_long_imdb_title()
        elif key == 'imdbID':
            return self.imdbID
        elif key == 'myTitle':
            return self.myTitle
        elif key == 'myID':
            return self.myID
        # XXX: should the key translation be done at the begin of this
        #      method?
        if key in self.keys_alias.keys():
            key = self.keys_alias[key]
        return self.__movie_data[key]

    def __nonzero__(self):
        """The Movie is "false" if the self.__movie_data is empty."""
        if self.__movie_data:
            return 1
        return 0

    def __cmp__(self, other):
        """Compare two Movie objects."""
        if not isinstance(other, self.__class__):
            return -1
        if self.__movie_data == other.__movie_data:
            return 0
        return 1

    # TODO: should methods like __contains__(), __delitem__() be defined?

    def copy(self):
        """Return a deep copy of the object itself."""
        return copy.deepcopy(self)

    def __str__(self):
        """Return a string with a pretty-printed summary for the movie."""
        if not self:
            return ''
        s = 'Movie\n=====\n'
        title = self.__movie_data.get('title')
        if title:
            s += 'Title: %s\n' % title.get_long_imdb_title()
        if self.imdbID:
            s += 'imdbID: %s\n' % self.imdbID
        if self.imdbURL:
            s += 'IMDb URL: %s\n' % self.imdbURL
        genres = self.__movie_data.get('genres')
        if genres:
            s += 'Genres: '
            for gen in genres:
                s += gen + ', '
            s = s[:-2] + '.\n'
        director = self.__movie_data.get('director')
        if director:
            s += 'Director: '
            for name in director:
                s += name + ', '
            s = s[:-2] + '.\n'
        writer = self.__movie_data.get('writer')
        if writer:
            s += 'Writer: '
            for name in writer:
                s += name + ', '
            s = s[:-2] + '.\n'
        cast = self.__movie_data.get('cast')
        if cast:
            cast = cast[:5]
            s += 'Cast: '
            for name in cast:
                s += name + ', '
            s = s[:-2] + '.\n'
        runtime = self.__movie_data.get('runtimes')
        if runtime:
            s += 'Runtime: '
            for r in runtime:
                s += r + ', '
            s = s[:-2] + '.\n'
        countries = self.__movie_data.get('countries')
        if countries:
            s += 'Country: '
            for c in countries:
                s += c + ', '
            s = s[:-2] + '.\n'
        lang = self.__movie_data.get('languages')
        if lang:
            s += 'Language: '
            for l in lang:
                s += l + ', '
            s = s[:-2] + '.\n'
        rating = self.__movie_data.get('rating')
        if rating:
            s += 'Rating: %s\n' % rating
        nr_votes = self.__movie_data.get('votes')
        if nr_votes:
            s += 'Votes: %s\n' % nr_votes
        plot = self.__movie_data.get('plot')
        if plot:
            plot = plot[0]
            i = plot.find('::')
            if i != -1:
                plot = plot[i+2:]
            s += 'Plot: %s' % plot
        return s


