"""
imdb package.

This package can be used to retrieve informations about a movie
from the IMDb database.
It can fetch data through different media (e.g.: the IMDb web pages,
the e-mail interface, a local installation, etc.)
*So far the only data access system available is though the
IMDb's web server. <g>*

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

__all__ = ['IMDb', 'IMDbError']


from imdb.utils import mixIn
from _exceptions import IMDbError


class IMDb:
    """The base class to search for a movie and to get a Movie object.
    
    This class cannot directly fetch data of any kind and so, when an
    object is instantiated, the class "mix-ins" itself with another
    class that defines the methods used to access the data through a given
    media."""
    def __init__(self, kind='http', *arguments, **keywords):
        """Initialize the IMDb class.
        
        *kind* -- a string representing the preferred data access system.
                    Valid values are:
                        'http': access though the web server (the default).
        *arguments and keywords* -- other arguments and keywords are
                                    ignore by the base class, but are
                                    passed to the optional _init() method
                                    of the mixed-in class."""
        # XXX: only access through the web server is implemented!
        if kind.lower() in ('http', 'web', 'html'):
            from parser.http import IMDbHTTPMixIn
            mixIn(self.__class__, IMDbHTTPMixIn)
        # XXX: as an example, this would be the syntax to implement
        #      a "local installation" data access system.
        #elif kind.lower() == 'local':
        #    from parser.local import IMDbLocalMixIn
        #    mixIn(self.__class__, IMDbLocalMixIn)
        # If not valid data access system is fond, raise an exception.
        else:
            raise IMDbError, \
                        'unknown kind of data access system: "%s"' % str(kind)
        self._init(*arguments, **keywords)

    def _init(self, *arguments, **keywords):
        """Could be overridden to initialize mix-ins classes."""
        pass

    def get_movie(self, uniqueID):
        """Return a Movie object for the given uniqueID; must be
        overridden by the mix-in class.
        
        The uniqueID is something used to univocally identify a movie;
        it can be the imdbID used by the IMDb web server, a file
        pointer, a line number in a file, an ID in a database, etc.
        """
        # XXX: for the real implementation, see the method of the mixed-in
        #      class, somewhere under the imdb.parser package.
        import Movie
        return Movie.Movie(imdbID=self.get_imdbID(uniqueID))

    def search_movie(self, title):
        """Return a SearchResults object for a query for the given title;
        must be overridden by the mix-in class.
        
        The SearchResults is a list of tuples like (uniqueID, MovieTitle)
        where "uniqueID" is everything that can be used by the get_movie()
        method to fetch the data for the given movie, and MovieTitle is
        an object instance of the imdb.dataSets.MovieTitle class.
        """
        # XXX: for the real implementation, see the method of the mixed-in
        #      class, somewhere under the imdb.parser package.
        import dataSets
        return dataSets.SearchResults([])

    def get_imdbID(self, uniqueID):
        """Translate a uniqueID in a imdbID (the ID used by the IMDb
        web server; must be overridden by the mix-in class."""
        # XXX: for the real implementation, see the method of the mixed-in
        #      class, somewhere under the imdb.parser package.
        return None

    def new_movie(self, imdbID=None, myTitle='', myID=None, movieData={}):
        """Return a Movie.Movie object."""
        # XXX: this method is not really useful...
        import Movie
        return Movie.Movie(imdbID, myTitle, myID, movieData)

    def get_special_methods(self):
        """Return the special methods defined by the mix-in class."""
        import types
        sm_dict = {}
        for name in dir(self.__class__):
            if name.startswith('_') or name in ('get_movie', 'search_movie',
                        'get_imdbID', 'new_movie', 'get_special_methods'):
                continue
            member = getattr(self.__class__, name)
            if type(member) is types.MethodType:
                sm_dict.update({name: member.__doc__})
        return sm_dict


