"""
dataSets module (imdb package).

This module provides the MovieTitle class, used by the Movie class to
store a complete IMDb movie title (e.g. "X-Files, The (1993) (TV)"), and
the SearchResults returned by a query made with the IMDb class.

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import re, types
from UserDict import UserDict
from UserList import UserList

from imdb._exceptions import IMDbInvalidTitle


# The regular expression for the "long" year format of IMDb, like
# "(1998)" and "1986/II", where the optional roman number (that I call
# "imdbIndex" after the slash is used for movies with the same title
# and year of release.
# XXX: probably X, L, C, D and M are far too much! ;-)
re_year_index = re.compile('\(([0-9\?]{4}(/[IVXLCDM]+)?)\)')


def analyze_title(title):
    """Analyze the given title and return a dictionary with the
    "stripped" title, the kind of the show ("movie", "tv serie", etc.),
    the year of production and the optional imdbIndex (a roman number
    used to distinguish between movies with the same title and year).
    
    raise an IMDbInvalidTitle exception if the title is not valid."""
    original_t = title
    title = title.strip()
    year = ''
    kind = ''
    imdbIndex = ''
    # First of all, search for the kind of show.
    if title.endswith('(mini)'):
        kind = 'tv mini serie'
        title = title[:-6]
    elif title.endswith('(TV)'):
        kind = 'tv movie'
        title = title[:-4]
    elif title.endswith('(V)'):
        kind = 'video movie'
        title = title[:-3]
    elif title.endswith('(VG)'):
        kind = 'video game'
        title = title[:-4]
    title = title.strip()
    # Search for the year and the optional imdbIndex (a roman number).
    yi = re_year_index.findall(title)
    if yi:
        last_yi = yi[-1]
        year = last_yi[0]
        if last_yi[1]:
            imdbIndex = last_yi[1][1:]
            year = year[:-len(imdbIndex)-1]
        i = title.rfind('(%s)' % last_yi[0])
        if i != -1:
            title = title[:i-1]
    title = title.strip()
    # This is a tv (mini) serie: strip the '"' at the begin and at the end.
    # XXX: strip('"') is not used for compatibility with Python 2.0.
    if title and title[0] == title[-1] == '"':
        if not kind:
            kind = 'tv serie'
        title = title[1:-1]
    title = title.strip()
    if not title:
        raise IMDbInvalidTitle, 'invalid title: "%s"' % str(original_t)
    result = {
                'title': title,                 # The 'real title'
                'year': year or '????',         # Release year.
                'kind': kind or 'movie',        # One of 'movie', 'tv serie',
                                                # 'tv mini serie',
                                                # 'tv movie', 'video movie',
                                                # 'video game'.
                'imdbIndex': imdbIndex or ''    # The II in '(1983/II)', for
                                                # multiple titles.
            }
    return result


def build_title(title_dict):
    """Given a dictionary that represents a "long" IMDb title,
    return a string."""
    title = title_dict.get('title', '')
    kind = title_dict.get('kind')
    imdbIndex = title_dict.get('imdbIndex')
    year = title_dict.get('year', '????')
    if kind in ('tv serie', 'tv mini serie'):
        title = '"%s"' % title
    title += ' (%s' % year
    if imdbIndex:
        title += '/%s' % imdbIndex
    title += ')'
    if kind == 'tv mini serie':
        title += ' (mini)'
    elif kind == 'tv movie':
        title += ' (TV)'
    elif kind == 'video movie':
        title += ' (V)'
    elif kind == 'video game':
        title += ' (VG)'
    return title.strip()


# XXX: this can be done in Python 2.2 inheriting from the 'dict' type.
class MovieTitle(UserDict):
    """Class used to store complete informations about a movie title."""
    valid_keys = ('title', 'year', 'imdbIndex', 'kind')
    def __init__(self, data={}):
        UserDict.__init__(self)
        self.data = {}
        # If a string is passed, it's parsed.
        if type(data) in (types.StringType, types.UnicodeType):
            data = analyze_title(data)
        for k, v in data.items():
            self.__setitem__(k, v)

    def __setitem__(self, key, value):
        # Only a key in valid_keys can be set.
        # XXX: obviously it's not bullet-proof, so don't count on it!
        if key not in self.valid_keys:
            return
        self.data.update({key: value})

    def __getitem__(self, key):
        return self.data[key]

    def get_long_imdb_title(self):
        """Return the movie title in the long format used by IMDb."""
        return build_title(self.data)

    def __str__(self):
        # Only print the title.
        return self.data['title']

    def __nonzero__(self):
        # Every valid_key should be set.
        for k in self.valid_keys:
            if not self.data.has_key(k):
                return 0
        return 1


# XXX: this can be done in Python 2.2 inheriting from the 'list' type.
class SearchResults(UserList):
    """The class used to store the results of a query."""
    def get_uniqueID_item(self, key):
        """Return the uniqueID of the key-th item."""
        return self.data[key][0]

    def get_title_item(self, key):
        """Return the MovieTitle object of the key-th item."""
        return self.data[key][1]

    def get_long_imdb_title(self, key):
        """Return the long IMDb title of the key-th item."""
        return self.data[key][1].get_long_imdb_title()

    def __str__(self):
        """Pretty print the results."""
        s = ''
        for uniqueID, title in self.data:
            s += '%s: %s\n' % (str(uniqueID), title.get_long_imdb_title())
        return s


