"""
parser.http package (imdb package).

This package provides the IMDbHTTPMixIn class that should be mixed
with the imdb.IMDb class to access IMDb's data through the web interface;
the IMDb will automatically "mix-ins" itself with this class when
instantiated with the 'kind' argument set to "http" or "web" or "html" (this
is the default).

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import urllib

from imdb.Movie import Movie
from imdb.dataSets import SearchResults
from imdb.utils import imdbURL_combdet, imdbURL_plot
from imdb._exceptions import IMDbDataAccessError, IMDbParserError
from imdb.parser.http.movieParser import movie_parser
from imdb.parser.http.plotParser import plot_parser
from imdb.parser.http.searchParser import search_parser


# XXX: you won't believe it, but it's not possibile to build the
# query string with urllib.urlencode() because the IMDb's "Find" CGI
# _wants_ the parameters in _this_ order. :-/
imdbURL_query_p1 = 'http://akas.imdb.com/Find?select=All+%28old%29&for='
imdbURL_query_p2 = '&Go.x=0&Go.y=0&Go=Go'


# FIXME: should the "open_unknown_proxy()", "open_unknown_proxy()" methods
#        be overridden?
class IMDbURLopener(urllib.FancyURLopener):
    """Fetch web pages and handle errors."""
    # XXX: does it make sense to override the http_error_default() method,
    #      or it's better to override only single methods, like
    #      http_error_404()?
    def http_error_default(self, url, fp, errcode, errmsg, headers):
        raise IMDbDataAccessError, {'url': 'http:%s' % url,
                                    'errcode': errcode,
                                    'errmsg': errmsg,
                                    'headers': headers}


class IMDbHTTPMixIn:
    """The Mix-ins class used to access IMDb's data through the web."""
    # The object used to access HTTP pages.
    urlOpener = IMDbURLopener()
    # XXX: IMDb's web server doesn't like crawlers, so let's fake
    #      to be Mozilla.
    #      Wow!  I'm shocked by my total lack of ethic! <g>
    urlOpener.addheaders = [('User-agent',
            'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.5) Gecko/20030917')]
    # The URL for the "combined details" and "plot summary" pages.
    imdbURL_combdet = imdbURL_combdet
    imdbURL_plot = imdbURL_plot

    def get_imdbID(self, uniqueID):
        """Translate a uniqueID in a imdbID; in this implementation
        the uniqueID _is_ the imdbID."""
        return uniqueID

    def set_proxy(self, proxy):
        """Set the web proxy to use.
        
        It should be a string like 'http://localhost:8080/'; if the
        string is empty, no proxy will be used.
        If set, the value of the environment variable HTTP_PROXY is used."""
        if not proxy:
            if self.urlOpener.proxies.has_key('http'):
                del self.urlOpener.proxies['http']
        else:
            if not proxy.lower().startswith('http://'):
                proxy = 'http://%s' % proxy
            self.urlOpener.proxies.update({'http': proxy})

    def get_proxy(self):
        """Return the used proxy or an empty string."""
        return self.urlOpener.proxies.get('http', '')

    def search_movie(self, title):
        """Search for a given title; return a SearchResults object.
        
        The uniqueID returned by the search_parser object is the
        imdbID used by the IMDb's web server.

        To do the query, it's used the "old search system" on the
        akas.imdb.com server; for the title "passion", the query is:
        http://akas.imdb.com/Find?select=All+%28old%29&for=passion&Go.x=0&Go.y=0&Go=Go
        """
        # The URL of the query.
        url = imdbURL_query_p1 + urllib.quote_plus(title) + imdbURL_query_p2
        try:
            query_url = self.urlOpener.open(url)
            search_result = query_url.read()
            query_url.close()
            self.urlOpener.close()
        except IOError, e:
            raise IMDbDataAccessError, {'errcode': e.errno,
                                        'errmsg': str(e.strerror),
                                        'url': imdbURL_combdet}
        # Return a SearchResults object.
        return SearchResults(search_parser.parse(search_result))

    def get_movie(self, uniqueID):
        """Return a Movie object for the given uniqueID."""
        # URL of the "combined details" and "plot summary" pages.
        imdbURL_combdet = self.imdbURL_combdet % uniqueID
        imdbURL_plot = self.imdbURL_plot % uniqueID
        try:
            cdu = self.urlOpener.open(imdbURL_combdet)
            md = cdu.read()
            cdu.close()
            self.urlOpener.close()
        except IOError, e:
            # XXX: do I need to include a finally clause to clean
            #      the state of the used objects?
            raise IMDbDataAccessError, {'errcode': e.errno,
                                        'errmsg': str(e.strerror),
                                        'url': imdbURL_combdet}
        pd = ''
        try:
            plu = self.urlOpener.open(imdbURL_plot)
            pd = plu.read()
            plu.close()
            self.urlOpener.close()
        except IOError, e:
            # XXX: I'm not sure: we can return a lot of informations
            #      about a movie even if we've not got the plot, but after
            #      all a serious trouble occurred (the IMDb web server show
            #      a web page even for movie without the "plot summary"
            #      section).
            pass
        # Parse "main details" and "plot summary" pages.
        movie_data = movie_parser.parse(md)
        if pd:
            try:
                plot_data = plot_parser.parse(pd)
            except IMDbParserError:
                # XXX: see above...
                pass
        else:
            plot_data = {}
        title = movie_data.get('title', '')
        # Merge the two dictionary returned by the parsers.
        movie_data.update(plot_data)
        movie = Movie(imdbID=uniqueID, myTitle=title, movieData=movie_data)
        # Return a Movie object.
        return movie


