"""
parser.http.movieParser module (imdb package).

This module provides the HTMLMovieParser class (and the movie_parser
instance), used to parse the "combined details" IMDb page on the
akas.imdb.com server.
E.g., for Brian De Palma's "The Untouchables", the referred
page would be 'http://akas.imdb.com/title/tt0094226/combined'

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import re, types
from htmllib import HTMLParser
from formatter import NullFormatter

from imdb.dataSets import MovieTitle
from imdb._exceptions import IMDbParserError, IMDbInvalidTitle
from imdb.parser.http.utils import get_attr_value


# Some regular expression.  See the places where they're used for comments.
RE_FLAGS = re.IGNORECASE ^ re.MULTILINE
re_rating = re.compile('([0-9][\.,][0-9])/10')
re_nr_votes = re.compile('\(([0-9\.,]+) votes\)')
re_runtime = re.compile('Runtime:</b>\n(.+)$', RE_FLAGS)
re_mpaa = re.compile('<a href="?/mpaa"?>MPAA</a>: ?</b>(.+?)<br>', RE_FLAGS)


def strip_amps(theString):
    """Remove '&\s*' at the end of a string.
    It's used to remove '& ' from strings like '(written by) & '."""
    # XXX: check if this function, done with a regexp, will
    #      save some execution time (I don't think, but...)
    i = theString.rfind('&')
    if i != -1:
        if not theString[i+1:] or theString[i+1:].isspace():
            # There's nothing except spaces after the '&'.
            theString = theString[:i]
    return theString


def clear_text(theString):
    """Remove separators and spaces in excess."""
    # XXX: this can be done with a couple of regexp, but I fear it
    #      will take much more computational time.
    # Squeeze multiple spaces into one.
    theString = ' '.join(theString.split())
    # Remove spaces around the '::' separator.
    theString = theString.replace(':: ', '::').replace(' ::', '::')
    # Remove exceeding '::' separators.  I love list comprehension! <g>
    theString = '::'.join([piece for piece in theString.split('::') if piece])
    return theString


# XXX: this class inherits from HTMLParser; see the documentation for
#      the "htmllib" and "sgmllib" modules.
class HTMLMovieParser(HTMLParser):
    """Parser for the "combined details" page of a given movie.
    The page should be provided as a string, as taken from
    the akas.imdb.com server.  The final result will be a
    dictionary, with a key for every relevant section.

    Example:
        mparser = HTMLMovieParser()
        result = mparser.parse(combined_details_html_string)
    """
    def __init__(self, formatter=NullFormatter(), verbose=0):
        # This is the dictionary that will be returned by the parse() method.
        self.__movie_data = {}
        HTMLParser.__init__(self, formatter, verbose)
        # Use a "normal" space in place of the non-breaking space (0240/0xA0).
        self.entitydefs['nbsp'] = ' '

    def reset(self):
        """Reset the parser."""
        HTMLParser.reset(self)
        self.__movie_data.clear()
        # XXX: this is quite a mess; the boolean variables below are
        #      used to identify what section of the HTML we're parsing,
        #      and the string variables are used to store temporary values.
        # The name of the section we're parsing.
        self.__current_section = ''
        # We're in a cast/crew section.
        self.__is_cast_crew = 0
        # We're managing the name of a person.
        self.__is_name = 0
        # The name and his role, sperated by '::'.
        self.__name = ''
        # We're in a company credit section.
        self.__is_company_cred = 0
        # The name of the company.
        self.__company_data = ''
        self.__countries = 0
        self.__is_genres = 0
        self.__is_title = 0
        self.__title = ''
        self.__is_rating = 0
        self.__rating = ''
        self.__is_languages = 0
        self.__is_akas = 0
        self.__aka_title = ''
        # If true, the next data should be merged with the previous one,
        # without the '::' separator.
        self.__merge_next = 0

    def append_item(self, sect, data):
        """Append a new value in the given section of the dictionary."""
        # Do some cleaning work on the strings.
        sect = clear_text(sect)
        data = clear_text(data)
        if not self.__movie_data.has_key(sect):
            self.__movie_data[sect] = []
        self.__movie_data[sect].append(data)

    def set_item(self, sect, data):
        """Put a single value (a string, normally) in the dictionary."""
        if type(data) in (types.StringType, types.UnicodeType):
            data = clear_text(data)
        self.__movie_data[clear_text(sect)] = data

    def get_movie_data(self):
        """Return the dictionary."""
        return self.__movie_data

    def start_title(self, attrs):
        self.__is_title = 1

    def end_title(self):
        if self.__title:
            self.__is_title = 0
            try:
                title = MovieTitle(self.__title)
            except IMDbInvalidTitle, e:
                raise IMDbParserError, e
            # title is a MovieTitle object.
            self.set_item('title', title)

    def start_table(self, attrs):
        pass

    def end_table(self):
        self.__is_cast_crew = 0
        self.__is_company_cred = 0
        self.__current_section = ''

    def start_a(self, attrs):
        # XXX: the current section ('director', 'cast', 'production company',
        #      etc.) name is taken from two different sources: for sections
        #      that contains actors/crew member names, the section name in
        #      the HTML is in the form
        #      <a href="/Glossary/S#SectName/">Description</a>, so we use
        #      the lowecase "sectname" as the name of the current section.
        #      For sections with company credits, the HTML is in the form
        #      <a href="/List?SectName=...">CompanyName</a> so we cut the
        #      href attribute of the <a> tag to get the "sectname".
        #      Underscores and minus signs in section names are replaced
        #      with spaces.
        link = get_attr_value(attrs, 'href')
        if not link:
            return
        link = link.lower()
        # Sections like 'cast', 'director', 'writer', etc. all
        # begin with a link to a "/Glossary" page.
        if link.startswith('/glossary'):
            self.__is_cast_crew = 1
            # Get the section name from the link.
            link = link[12:].replace('_',' ').replace('-',' ')
            self.__current_section = link
        elif link.startswith('/name'):
            # The following data will be someone's name.
            self.__is_name = 1
        elif link.startswith('/list'):
            self.__is_company_cred = 1
            # Sections like 'company credits' all begin
            # with a link to "/List" page.
            sect = link[6:].replace('_', ' ').replace('-', ' ')
            # The section name ends at the first '='.
            ind = sect.find('=')
            if ind != -1:
                sect = sect[:ind]
            self.__current_section = sect
        elif link.startswith('/sections/countries'):
            self.__countries = 1
        elif link.startswith('/sections/genres'):
            self.__is_genres = 1
        elif link.startswith('/sections/languages'):
            self.__is_languages = 1

    def end_a(self):
        pass

    def start_tr(self, attrs):
        pass

    def end_tr(self):
        if self.__is_name and self.__current_section:
            # Most of the cast/crew informations are separated by <tr> tags.
            if self.__is_cast_crew:
                # Strip final '&' (and spaces); this is here
                # to get rid of things like the "& " in "(novel) & ".
                self.__name = strip_amps(self.__name)
                self.append_item(self.__current_section, self.__name)
            self.__name = ''
        self.__is_name = 0

    def do_br(self, attrs):
        if self.__is_company_cred:
            # Sometimes companies are separated by <br> tags.
            self.__company_data = strip_amps(self.__company_data)
            self.append_item(self.__current_section, self.__company_data)
            self.__company_data = ''
            self.__is_company_cred = 0

    def start_li(self, attrs):
        pass

    def end_li(self):
        if self.__is_company_cred:
            # Sometimes companies are listed inside an <ul> tag.
            self.__company_data = strip_amps(self.__company_data)
            self.append_item(self.__current_section, self.__company_data)
            self.__company_data = ''
            self.__is_company_cred = 0

    def start_ul(self, attrs):
        pass

    def end_ul(self):
        self.__is_company_cred = 0

    def start_b(self, attrs):
        self.__is_akas = 0

    def end_b(self):
        pass

    def start_i(self, attrs):
        pass

    def end_i(self):
        if self.__is_akas:
            # XXX: when managing an 'aka title', some transformation
            #      are required; a complete aka title in the form
            #      of "Aka Title (year) (country1) (country2) [cc]"
            #      will be stripped of " (year) " if it's equal to
            #      the movie's year of release; the "Aka Title", the
            #      countries list and the country code (cc) will be
            #      separated by '::'.
            #      In the example: "Aka Title::(country1) (country2)::[cc]"
            aka = self.__aka_title
            year = None
            title = self.__movie_data.get('title')
            if title:
                year = title.get('year')
            if year:
                aka = aka.replace(' (' + year + ') ', '::')
            ind = aka.rfind(' [')
            if ind != -1:
                aka = aka[:ind] + '::' + aka[ind+1:]
            self.append_item('akas', aka)
            self.__aka_title = ''

    def start_body(self, attrs):
        pass

    def end_body(self):
        # FIXME: some horrible, horrible hacks. :-/
        # Find the runtime using a regular expression on the
        # whole page; the same is done for the MPAA rating.
        # This can be a source of troubles, because the regular
        # expression engine (written in C) can completely fill up
        # the stack, causing a RuntimeError (maximum recursion limit
        # exceeded) doing a non-greedy matching upon a too long string.
        # Summary: use less regular expression as possible (parse html
        # with regexp is generally a really bad idea), try to avoid
        # non-greedy regexp or - at least - try to do the match upon a
        # string as short as possibile.
        rt = re_runtime.findall(self.rawdata)
        if rt:
            rt = rt[0].replace('min', '')
            runtimes = rt.split()
            for runtime in runtimes:
                self.append_item('runtimes', runtime)
        mpaas = re_mpaa.findall(self.rawdata)
        # XXX: I _think_ that the MPAA rating can be a multiple
        #      value; can anyone point me to a movie with multiple ratings?
        for mpaa in mpaas:
            self.append_item('mpaa', mpaa)

    def do_img(self, attrs):
        alttex = get_attr_value(attrs, 'alt')
        if not alttex:
            return
        alttex = alttex.lower().strip()
        if alttex == 'cover':
            # Get the URL of the cover image.
            src = get_attr_value(attrs, 'src')
            if src:
                self.set_item('cover url', src)
        elif alttex in ('*', '_'):
            # The gold and grey stars; we're near the rating and number
            # of votes.
            self.__is_rating = 1
        elif alttex == 'vote here':
            # Parse the rating and the number of votes.
            self.__is_rating = 0
            # XXX: see comments for the end_body() method!
            rating = re_rating.findall(self.__rating)
            if rating:
                self.set_item('rating', rating[0])
            nr_votes = re_nr_votes.findall(self.__rating)
            if nr_votes:
                self.set_item('votes', nr_votes[0])

    def handle_charref(self, name):
        # This methods is the same found in sgmllib.py, except
        # that it doesn't call handle_data(), but manage the
        # entity internally.
        try:
            n = int(name)
        except ValueError:
            self.unknown_charref(name)
            return
        if not 0 <= n <= 255:
            self.unknown_charref(name)
            return
        c = chr(n)
        self.simplyAdd(c)

    def handle_entityref(self, name):
        # See comments for the handle_charref() methods.
        table = self.entitydefs
        if table.has_key(name):
            self.simplyAdd(table[name])
        else:
            self.unknown_entityref(name)
            return
    
    def simplyAdd(self, c):
        # Append an entity to the string we're building.
        # It's used by handle_charref() and handle_entityref() methods.
        if self.__is_company_cred and self.__current_section:
            self.__company_data += c
            if c != ' ':
                # XXX: sigh!  This is here to catch &nbsp; (replaced with
                #      a space); sometimes it's used to begin the description
                #      of the role of a given company.
                #      I'm not sure this will work correctly everywhere...
                self.__merge_next = 1
        elif self.__is_cast_crew and self.__current_section and self.__is_name:
            self.__name += c
        elif self.__is_title:
            self.__title += c

    def handle_data(self, data):
        # Manage the plain text part of an HTML document.
        if self.__is_cast_crew and self.__current_section and self.__is_name:
            # Modify the separator for "name ... role"
            data = data.replace(' .... ', '::')
            # Strip everything starting from the last ')'.
            # XXX: is this useful?  doesn't it cut useful informations?
            ri = data.rfind(')')
            if ri != -1:
                data = data[:ri+1]
            self.__name += data
        elif self.__is_company_cred and self.__current_section:
            # Sometimes company credits are separated by a slash;
            # 'certification' is an example.
            if data.strip() == '/':
                cd = self.__company_data
                cd = strip_amps(cd)
                self.append_item(self.__current_section, cd)
                self.__company_data = ''
                self.__is_company_cred = 0
            else:
                # Merge the next data without a separator.
                if self.__merge_next:
                    self.__company_data += data
                    self.__merge_next = 0
                else:
                    if self.__company_data:
                        self.__company_data += '::'
                    self.__company_data += data
        elif self.__is_akas and data.strip() != ':':
            self.__aka_title += data
        elif data.strip() == 'Also Known As':
            self.__is_akas = 1
        elif self.__countries:
            self.append_item('countries', data)
            self.__countries = 0
        elif self.__is_title:
            # Store the title and the year, as taken from the <title> tag.
            self.__title += data
        elif self.__is_genres:
            self.append_item('genres', data)
            self.__is_genres = 0
        elif self.__is_rating:
            self.__rating += data
        elif self.__is_languages:
            self.append_item('languages', data)
            self.__is_languages = 0

    def error(self, message):
        raise IMDbParserError, 'SGML parser error: "%s"' % str(message)

    def parse(self, html_string):
        """Return the dictionary generated from the given html string."""
        self.reset()
        self.feed(html_string)
        return self.get_movie_data()


# The used instance.
movie_parser = HTMLMovieParser()


