"""
parser.http.searchParser module (imdb package).

This module provides the HTMLSearchParser class (and the search_parser
instance), used to parse the results of a search for a given title.
It parses the IMDb's old search system, because it provides much
cleaner informations.

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import re
from htmllib import HTMLParser
from formatter import NullFormatter

from imdb.dataSets import MovieTitle
from imdb._exceptions import IMDbParserError, IMDbInvalidTitle
from imdb.parser.http.utils import get_attr_value

# The imdbID is a 7-ciphers number.
re_imdbID = re.compile('([0-9]{7})')


# XXX: this class inherits from HTMLParser; see the documentation for
#      the "htmllib" and "sgmllib" modules.
class BasicMovieParser(HTMLParser):
    """Simply get the title of a movie and the imdbID.
    
    It's used by the HTMLSearchParser class to return a result
    for a direct match (when a search on IMDb results in a single
    movie, the web server sends directly the movie page."""
    # XXX: should this class be moved into the movieParser.py file?
    def __init__(self, formatter=NullFormatter(), verbose=0):
        HTMLParser.__init__(self, formatter, verbose)
        self.entitydefs['nbsp'] = ' '

    def reset(self):
        """Reset the parser."""
        HTMLParser.reset(self)
        self.__result = []
        self.__reading_page_title = 0
        self.__page_title = ''
        self.__last_imdbID = ''

    def get_result(self):
        """Return a list with a single tuple ('uniqueID', MovieTitle())
        where uniqueID is the imdbID."""
        return self.__result

    def start_title(self, attrs):
        self.__reading_page_title = 1

    def end_title(self):
        self.__reading_page_title = 0

    def start_a(self, attrs):
        link = get_attr_value(attrs, 'href')
        if link and link.lower().startswith('/title'):
            imdbID = re_imdbID.findall(link)
            if imdbID:
                self.__last_imdbID = imdbID[-1]

    def end_a(self):
        self.__last_imdbID = ''

    def do_img(self, attrs):
        # XXX: here we're searching for the "pageflicker" links; the
        #      one around the "main details" image (the square/stop icon)
        #      is the only reliable source to read the imdbID from...
        alttext = get_attr_value(attrs, 'alt')
        if alttext and self.__last_imdbID and self.__page_title \
                and alttext.strip().lower() == 'main details':
            # we got the title, and we got the imdbID.
            title = self.__page_title.strip()
            try:
                res = [(self.__last_imdbID,
                        MovieTitle(title))]
            except IMDbInvalidTitle, e:
                raise IMDbParserError, e
            self.reset()
            self.__result = res

    def handle_data(self, data):
        if self.__reading_page_title:
            self.__page_title += data

    def error(self, message):
        raise IMDbParserError, 'SGML parser error: "%s"' % str(message)

    def parse(self, movie_html_page):
        """Parse the given HTML "main details" page for a movie."""
        self.reset()
        self.feed(movie_html_page)
        return self.get_result()


# XXX: this class inherits from HTMLParser; see the documentation for
#      the "htmllib" and "sgmllib" modules.
class HTMLSearchParser(HTMLParser):
    """Parse the html page that the IMDb web server shows when the
    "old search system" is used."""
    # TODO: the only results that are returned, are movie titles (so
    #       search for names of actors or directors will not work).
    def __init__(self, formatter=NullFormatter(), verbose=0):
        HTMLParser.__init__(self, formatter, verbose)
        self.entitydefs['nbsp'] = ' '

    def reset(self):
        """Reset the parser."""
        HTMLParser.reset(self)
        self.__results = []
        self.__begin_list = 0
        self.__is_title = 0
        self.__reading_page_title = 0
        self.__current_imdbID = ''
        self.__current_title = ''

    def get_results(self):
        """Return a list of ('imdbID', MovieTitle()) tuples."""
        return self.__results

    def start_title(self, attrs):
        self.__reading_page_title = 1

    def end_title(self):
        self.__reading_page_title = 0

    def start_ol(self, attrs):
        pass

    def end_ol(self):
        self.__begin_list = 0
        self.__is_title = 0
        self.__current_title = ''
        self.__current_imdbID = ''

    def start_li(self, attrs):
        pass

    def end_li(self):
        self.__is_title = 0

    def start_a(self, attrs):
        link = get_attr_value(attrs, 'href')
        # The next data is a movie title; now store the imdbID.
        if link and link.lower().startswith('/title'):
            nr = re_imdbID.findall(link[6:])
            if not nr:
                return
            self.__current_imdbID = nr[0]
            self.__is_title = 1

    def end_a(self):
        if self.__begin_list and self.__is_title:
            # We should have got the title.
            title = self.__current_title.strip()
            try:
                tup = (self.__current_imdbID, MovieTitle(title))
                # Check if the result is not already in the list.
                if tup not in self.__results:
                    self.__results.append(tup)
            except IMDbInvalidTitle, e:
                raise IMDbParserError, e
        self.__current_title = ''
        self.__current_imdbID = ''
        self.__is_title = 0

    def handle_data(self, data):
        if self.__begin_list and self.__is_title:
            self.__current_title += data
        elif self.__reading_page_title:
            if data.strip().lower() != 'imdb name and title search':
                # XXX: a direct result!
                #      Interrupt parsing, and retrieve data using a
                #      BasicMovieParser object.
                rawdata = self.rawdata
                self.reset()
                # Get imdbID and title directly from the "main details" page.
                bmp = BasicMovieParser()
                self.__results = bmp.parse(rawdata)
        else:
            # XXX: we have to check the plain text part of the HTML
            #      to know when the list of title begins.
            data = data.strip().lower()
            if data.startswith('most popular title searches') or \
                    data.startswith('\'title\' matches (i.e. movies, tv,' + \
                                    ' video etc)'):
                self.__begin_list = 1

    def error(self, message):
        raise IMDbParserError, 'SGML parser error: "%s"' % str(message)

    def parse(self, html_res_page):
        """Parse the given html page (as built by the "old search system")
        and return a list of ('imdbID', MovieTitle()) tuples."""
        self.reset()
        self.feed(html_res_page)
        return self.get_results()


# The used object.
search_parser = HTMLSearchParser()


